package fr.iut.sciencequest.viewModels

import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import fr.iut.sciencequest.viewModels.uiStates.PenduUIState
import fr.iut.sciencequest.model.repositories.scientifique.IScientifiqueRepository
import fr.iut.sciencequest.model.repositories.scientifique.ScientifiqueAPIRepository
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch

class PenduViewModel(
    val scientifiqueRepo: IScientifiqueRepository
) : ViewModel() {
    private val _uiState = MutableStateFlow(PenduUIState())
    val uiState = _uiState.asStateFlow()

    fun InitPartie() {
        viewModelScope.launch {
            scientifiqueRepo.fetchScientifiqueById(1)
            val scientifique = scientifiqueRepo.scientifique.value
            val nomComplet = scientifique.prenom + " " + scientifique.nom
            var motATrou = ""
            for (chr in nomComplet) {
                motATrou += if (chr == ' ') {
                    ' '
                } else {
                    '_'
                }
                _uiState.value = PenduUIState(
                    isActionGood = true,
                    motATrouver = nomComplet,
                    motATrou = motATrou
                )
            }
        }
    }

    // mot : mot à trouver
    // motAct : état actuel du mot trouvé par l'utilisateur
    fun PlayAction(lettre: Char) {
        val lowerCaseLetter = lettre.lowercaseChar()
        if (lettre == ' ' || _uiState.value.lettresUtilises.contains(lowerCaseLetter)) {
            _uiState.value = PenduUIState(false,
                false,
                _uiState.value.nbViesRestantes,
                _uiState.value.motATrouver,
                _uiState.value.motATrou,
                _uiState.value.lettresUtilises
            )
            return
        } else if (_uiState.value.motATrouver.lowercase().contains(lowerCaseLetter)) {
            var nvMotATrou = _uiState.value.motATrou
            for (index in _uiState.value.motATrouver.indices) {
                val letterToCheck = _uiState.value.motATrouver[index]
                if (letterToCheck.lowercaseChar() == lowerCaseLetter) {
                    nvMotATrou = nvMotATrou.replaceRange(index,index + 1, letterToCheck.toString())
                }
            }

            val isWon = nvMotATrou == _uiState.value.motATrouver

            _uiState.value = PenduUIState(isWon,
                true,
                _uiState.value.nbViesRestantes,
                _uiState.value.motATrouver,
                nvMotATrou,
                _uiState.value.lettresUtilises.plus(lowerCaseLetter)
            )
        } else {
            _uiState.value = PenduUIState(false,
                true,
                _uiState.value.nbViesRestantes - 1,
                _uiState.value.motATrouver,
                _uiState.value.motATrou,
                _uiState.value.lettresUtilises.plus(lowerCaseLetter)
            )
        }
    }
    companion object {

        val ApiFactory: ViewModelProvider.Factory = object : ViewModelProvider.Factory {
            @Suppress("UNCHECKED_CAST")
            override fun <T : ViewModel> create(
                modelClass: Class<T>
            ): T {
                return PenduViewModel(
                    ScientifiqueAPIRepository()
                ) as T
            }
        }
    }
}